/***************************************************************************************
子機XBeeのスイッチ状態をリモートで取得する

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <LiquidCrystal.h>
#define FORCE_INTERVAL  250                     // データ要求間隔(およそ10msの倍数)

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte dev[8];                                    // XBee子機のアドレス保存用
byte trig=0xFF;                                 // 子機へデータ要求するタイミング調整用

void setup() {
    lcd.begin(16, 2);                           // LCDのサイズを16文字×2桁に設定
    lcd.clear();                                // 画面消去
    lcd.print("Example 39 SW_F ");              // 文字を表示
    xbee_init( 0 );                             // XBee用COMポートの初期化
    xbee_atnj( 0xFF );                          // 子機XBeeデバイスを常に参加受け入れ
    lcd.setCursor(0, 1);                        // LCDのカーソル位置を2行目の先頭に
    lcd.print("Waiting for XBee");              // 待ち受け中の表示
}

void loop() {
    XBEE_RESULT xbee_result;                    // 受信データ(詳細)
    
    if( trig == 0){
        xbee_force( dev );                      // 子機へデータ要求を送信
        trig = FORCE_INTERVAL;
    }
    if( trig != 0xFF ) trig--;                  // 変数trigが0xFF以外の時に値を1減算

    xbee_rx_call( &xbee_result );               // データを受信
    switch( xbee_result.MODE ){                 // 受信したデータの内容に応じて
        case MODE_RESP:                         // xbee_forceに対する応答の時
            lcd.setCursor(0, 0);
            lcd.print("D1:");
            lcd.print(xbee_result.GPI.PORT.D1); // 子機XBeeのポート1の状態を表示する
            lcd.print(" D2:");
            lcd.print(xbee_result.GPI.PORT.D2); // 子機XBeeのポート2の状態を表示する
            lcd.print(" D3:");
            lcd.print(xbee_result.GPI.PORT.D3); // 子機XBeeのポート3の状態を表示する
            lcd.print("  ");
            break;
        case MODE_IDNT:                         // 新しいデバイスを発見
            lcd.setCursor(0, 1);
            lcd.print("Found a Device  ");
            for(byte i=0;i<8;i++)dev[i]=xbee_result.FROM[i];    // アドレスをdevにコピー
            xbee_atnj(0);                       // 親機XBeeに子機の受け入れ拒否を設定
            xbee_ratnj(dev,0);                  // 子機に対して孫機の受け入れ拒否を設定
            xbee_gpio_config(dev,1,DIN);        // 子機XBeeのポート1をデジタル入力に
            xbee_gpio_config(dev,2,DIN);        // 子機XBeeのポート2をデジタル入力に
            xbee_gpio_config(dev,3,DIN);        // 子機XBeeのポート3をデジタル入力に
            trig = 0;                           // 子機へデータ要求を開始
            break;
    }
}
