/***************************************************************************************
自作ブレッドボードセンサで照度測定を行う

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <LiquidCrystal.h>
#define FORCE_INTERVAL  250                     // データ要求間隔(およそ10msの倍数)

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte dev[8];                                    // XBee子機のアドレス保存用
byte trig=0xFF;                                 // データ要求するタイミング調整用

void set_ports(byte *dev){
    xbee_gpio_config( dev, 1 , AIN );           // XBee子機のポート1をアナログ入力へ
    xbee_end_device( dev, 1, 0, 0);             // 起動間隔1秒,自動測定無効
}

void setup() {
    lcd.begin(16, 2);                           // LCDのサイズを16文字×2桁に設定
    lcd.clear();                                // 画面消去
    lcd.print("Example 44 SensF");              // 文字を表示
    xbee_init( 0 );                             // XBee用COMポートの初期化
    xbee_atnj( 0xFF );                          // 親機に子機のジョイン許可を設定
    lcd.setCursor(0, 1);                        // LCDのカーソル位置を2行目の先頭に
    lcd.print("Waiting for XBee");              // 待ち受け中の表示
}

void loop() {
    byte i;
    float value;                                // 受信データの代入用
    XBEE_RESULT xbee_result;                    // 受信データ(詳細)
    
    /* データ送信 */
    if( trig == 0){
        xbee_force( dev );                      // 子機へデータ要求を送信
        trig = FORCE_INTERVAL;
    }
    if( trig != 0xFF ) trig--;                  // 変数trigが0xFF以外の時に値を1減算

    /* データ受信(待ち受けて受信する) */
    xbee_rx_call( &xbee_result );               // データを受信
    switch( xbee_result.MODE ){                 // 受信したデータの内容に応じて
        case MODE_RESP:                         // xbee_forceに対する応答の時
            // 照度測定結果をvalueに代入してprintfで表示する
            value = (float)xbee_result.ADCIN[1] * 3.55;
            lcd.clear();
            lcd.print(value, 1);
            lcd.print(" Lux");
            break;
        case MODE_IDNT:                         // 新しいデバイスを発見
            lcd.clear();
            lcd.print("Found New Device");
            for(i=0;i<8;i++) dev[i]=xbee_result.FROM[i];    // アドレスをdevにコピー
            xbee_atnj(0);                       // 親機XBeeに子機の受け入れ拒否
            xbee_ratnj(dev,0);                  // 子機に対して孫機の受け入れを拒否
            set_ports( dev );                   // 子機のGPIOポートの設定
            trig = 0;                           // 子機へデータ要求を開始
            break;
    }
}
