/***************************************************************************************
玄関が明るくなったらリビングの家電をONにする

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <LiquidCrystal.h>
#define FORCE_INTERVAL  250                     // データ要求間隔(およそ10msの倍数)

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte dev[8];                                    // XBee子機のアドレス保存用
byte dev_sens[8];                               // XBee子機(自作センサ)のアドレス
byte dev_plug[8];                               // XBee子機(Smart Plug)のアドレス
byte trig=0xFF;                                 // データ要求するタイミング調整用

byte set_ports(byte *dev, byte *dev_sens, byte *dev_plug){
    byte i;
    if( xbee_ping( dev ) != DEV_TYPE_PLUG){     // 自作センサの時(SmartPlugではない時)
        lcd.clear();
        lcd.print("Found a Sensor");            // XBee子機(自作センサ)の発見表示
        for(i=0;i<8;i++) dev_sens[i]=dev[i];    // アドレスをdevからdev_sensにコピー
        xbee_gpio_config( dev_sens, 1 , AIN );  // XBee子機のポート1をアナログ入力へ
        xbee_end_device( dev_sens, 3, 3, 0);    // 起動間隔3秒,自動測定3秒,SLEEP端子無効
        return( DEV_TYPE_XBEE );
    }else{                                      // 純正Smart Plugの時
        lcd.clear();
        lcd.print("Found a Plug");              // XBee子機(純正Smart Plug)の発見表示
        for(i=0;i<8;i++) dev_plug[i]=dev[i];    // アドレスをdevからdev_plugにコピー
        xbee_ratnj(dev_plug,5);                 // 子機に対して孫機の受け入れを拒否
        xbee_gpio_config(dev_plug, 1 , AIN );   // XBee子機のポート1をアナログ入力AINへ
        xbee_gpio_config(dev_plug, 2 , AIN );   // XBee子機のポート2をアナログ入力AINへ
        xbee_gpio_config(dev_plug, 3 , AIN );   // XBee子機のポート3をアナログ入力AINへ
        xbee_gpio_config(dev_plug, 4 , DOUT_H); // XBee子機のポート3をデジタル出力へ
        return( DEV_TYPE_PLUG );
    }
}

void setup() {
    byte dd = 0x00;                             // XBee子機の種類
    lcd.begin(16, 2);                           // LCDのサイズを16文字×2桁に設定
    lcd.clear();                                // 画面消去
    lcd.print("Example 48-2 Plg");              // 文字を表示
    xbee_init( 0 );                             // XBee用COMポートの初期化
    lcd.setCursor(0, 1);                        // LCDのカーソル位置を2行目の先頭に
    lcd.print("Waiting for XBee");              // 待ち受け中の表示
    lcd.setCursor(0, 1);                        // LCDのカーソル位置を2行目の先頭に
    while( dd != DEV_TYPE_PLUG ){               // Smart Plugが見つかるまで繰り返す
        if(xbee_atnj(30) != 0){                 // デバイスの参加受け入れを開始
            xbee_from( dev );                   // 見つけた子機のアドレスを変数devへ
            dd=set_ports(dev,dev_sens,dev_plug);    // 見つけた子機のポート設定を実行
        }
    }
    lcd.setCursor(0, 1);                        // LCDのカーソル位置を2行目の先頭に
    lcd.print("Waiting for XBee");              // 待ち受け中の表示
    xbee_atnj(0xFF);                            // 常にジョイン許可に設定
}

void loop() {
    byte i;
    float value;                                // 受信データの代入用
    XBEE_RESULT xbee_result;                    // 受信データ(詳細)
    
    if( trig == 0){
        xbee_force( dev_plug );                 // Smart Plugへデータ要求を送信
        trig = FORCE_INTERVAL;
    }
    trig--;
    
    xbee_rx_call( &xbee_result );               // データを受信
    switch( xbee_result.MODE ){                 // 受信したデータの内容に応じて
        case MODE_GPIN:                         // 子機XBeeの自動送信の受信
            if( bytecmp(xbee_result.FROM,dev_sens,8) == 0 ){ // 自作センサの受信
                value = (float)xbee_result.ADCIN[1] * 0.391;
                lcd.setCursor(0, 0);            // LCDのカーソル位置を1行目の先頭に
                lcd.print("SENS:");
                lcd.print(value,0);
                lcd.print(" Lux ");
                if( value > 10 ){               // 照度が10Lux以下の時
                    xbee_gpo(dev_plug , 4 , 1); // 純正Smart Plugのポート4をHに設定
                }else{
                    xbee_gpo(dev_plug , 4 , 0); // 純正Smart Plugのポート4をLに設定
                }
                xbee_force( dev_plug );         // 純正Smart Plugの照度取得指示
            }
            break;
        case MODE_RESP:                         // xbee_forceに対する応答の時
            if( bytecmp(xbee_result.FROM,dev_plug,8) == 0 ){ // Smart Plugの受信
                value = xbee_sensor_result( &xbee_result,WATT);
                lcd.setCursor(0, 1);            // LCDのカーソル位置を2行目の先頭に
                lcd.print("PLUG:");
                lcd.print(value,0);
                lcd.print(" W ");
                if( xbee_gpi(dev_plug ,4)==0 ){ // ポート4の状態を読みとり、
                    lcd.print("OFF ");          // 0の時はOFFと表示する
                }else{
                    lcd.print("ON ");           // 1の時はONと表示する
                }
            }
            break;
        case MODE_IDNT:                         // 新しいデバイスを発見
            lcd.clear();
            lcd.print("Found New Device");
            for(i=0;i<8;i++) dev[i]=xbee_result.FROM[i];    // アドレスをdevにコピー
            set_ports(dev,dev_sens,dev_plug);   // 見つけた子機のポート設定
            break;
    }
}
