/***************************************************************************************
湿度を測定してXBee親機に送信するXBee子機の実験用サンプルです。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <LiquidCrystal.h>

#define FORCE_INTERVAL  250                         // データ送信間隔(およそ10msの倍数)
#define TEMP_OFFSET     0.0                         // Arduino内部温度上昇の補正用

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
byte dev[]={0,0,0,0,0,0,0,0};                       // XBee親機アドレス（全てゼロ）
byte trig=0;                                        // 測定待ち用

void setup(){
    HS15Init();
    lcd.begin(16, 2);                               // LCDの初期化
    lcd.clear(); lcd.print("Example 58 Hum");
    xbee_init( 0x00 );                              // XBeeの初期化
    xbee_atcb( 4 );                                 // ネットワーク初期化
    xbee_atnj(0);                                   // 孫機に対するジョイン拒否
    lcd.setCursor(0, 1);                            // 液晶の２行目に移動
    lcd.print("Commissioning");                     // 文字を表示
    while( xbee_atai() > 0x01 ){                    // ネットワーク参加状況を確認
        delay(3000);
        xbee_atcb(1);                               // ネットワーク参加ボタンを押下
    }
    lcd.clear();
}

void loop(){
    byte i;
    char s[17];                                     // シリアルデータ保存用
    XBEE_RESULT xbee_result;                        // 受信データ用
    
    if( trig == 0){
        float hum;                                  // 湿度値用の変数
        float temp=getTemp()-TEMP_OFFSET;
        hum = HS15Read( temp );                     // 湿度の読み取り
        lcd.clear();
        lcd.print("TEMP=");
        lcd.print(temp,0);                          // 温度表示
        lcd.print("C ");
        lcd.setCursor(0,1);
        lcd.print("HUM =");
        lcd.print(hum,0);                           // 湿度表示
        lcd.print("% ");
        sprintf(s,"%02d,%02d\r",(int)temp,(int)hum);// 送信データ作成
        xbee_uart(dev,s);                           // データを送信
        trig = FORCE_INTERVAL;
    }
    trig--;                                         // 自動送信しない場合はこの行を削除
    
    xbee_rx_call( &xbee_result );
    if( xbee_result.MODE == MODE_UART ){            // シリアルデータを受信したとき
        if(xbee_result.DATA[0] == 0x1B){            // ESCコードが先頭だったとき
            for(i=0;i<16;i++){
                dev[i]=xbee_result.FROM[i];         // 送信元をdevに保存
            }
            trig=0;                                 // 次回のloopで測定して送信
        }
    }
}
