/*********************************************************************

                    HS-15Pの簡易湿度計 for Arduino

本ソースリストおよびソフトウェアは、ライセンスフリーです。
個人での利用は自由に行えます。著作権表示の改変は禁止します。

                               Copyright (c) 2012-2013 Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/

関連ページ：
    http://www.geocities.jp/bokunimowakaru/diy/arduino/humidity.html

接続方法：
   HS-15PをArduinoのANALOG 5とANALOG 3に接続します。
   また、ANALOG 3とGNDの間に0.022μFのコンデンサを挿入します。

再配布については下記の著作者の承諾を得る必要があります。

    HS15湿度センサーと簡易回路による湿度計実験
    渡波 郁
    http://homepage3.nifty.com/sudamiyako/zk/hs15/hs15.htm
    
*********************************************************************/
/*
参考文献

本ソフトの開発のために下記の文献を参考にしました。

・HS15湿度センサーと簡易回路による湿度計実験
    渡波郁  http://homepage3.nifty.com/sudamiyako/zk/hs15/hs15.htm
・Arduinoをはじめよう (オライリージャパン)
    Massimo Banzi著 船田功訳
・Arduino 日本語リファレンス
    http://www.musashinodenpa.com/arduino/ref/index.php
*/

#define HS15_P_OUT      19                      // ポート番号(ANALOG 5)
#define HS15_P_IN       3                       // ADCポート番号(ANALOG 3)

void HS15Init(void){
    pinMode(HS15_P_OUT, OUTPUT);                //出力ポート動作へ
    digitalWrite(HS15_P_OUT, LOW);
}

int HS15GetRs(const unsigned int ChgTime)
{
    int j;

    /* 測定準備。C放電。*/
    pinMode(HS15_P_OUT, OUTPUT);                // 出力ポート動作へ
    digitalWrite(HS15_P_OUT, LOW);
    for(j=0;j<20;j++){                          // 200mSec又は放電完了まち
        delay( 10 );                            // 10mSec
        if( analogRead(HS15_P_IN) < 5 )break;   // 放電完了をチェックし、ＯＫならば打ち切る
    }
    if( analogRead(HS15_P_IN) > 4 ){
        Serial.print( "leaves some charges = 0x" );
        Serial.println( analogRead(HS15_P_IN) , HEX );
        delay(3000);
    }
    
    /* 測定 */
    noInterrupts();
    if( ChgTime == 5 ){                         //ChgTimeはチャージ時間の半分の値
        digitalWrite(HS15_P_OUT, HIGH);
        delayMicroseconds( 10 );                // 充電 10us
        pinMode(HS15_P_OUT, INPUT);             //ハイインピーダンスへ(入力ポート)
        digitalWrite(HS15_P_OUT, LOW);          //プルアップOFF
        j = analogRead(HS15_P_IN);              //A/D 10bit
    }else if( ChgTime == 50 ){
        digitalWrite(HS15_P_OUT, HIGH);
        delayMicroseconds( 100 );               // 充電 100us
        pinMode(HS15_P_OUT, INPUT);             //ハイインピーダンスへ(入力ポート)
        digitalWrite(HS15_P_OUT, LOW);          //プルアップOFF
        j = analogRead(HS15_P_IN);              //A/D 10bit
    }else if( ChgTime == 500 ){
        digitalWrite(HS15_P_OUT, HIGH);
        delayMicroseconds( 1000 );              // 充電 1ms
        pinMode(HS15_P_OUT, INPUT);             //ハイインピーダンスへ(入力ポート)
        digitalWrite(HS15_P_OUT, LOW);          //プルアップOFF
        j = analogRead(HS15_P_IN);              //A/D 10bit
    }else if( ChgTime == 5000 ){
        digitalWrite(HS15_P_OUT, HIGH);
        delay( 10 );                            // 充電 10ms
        pinMode(HS15_P_OUT, INPUT);             //ハイインピーダンスへ(入力ポート)
        digitalWrite(HS15_P_OUT, LOW);          //プルアップOFF
        j = analogRead(HS15_P_IN);              //A/D 10bit
    }else{
        digitalWrite(HS15_P_OUT, HIGH);
        delay( 100 );                           // 充電 100ms
        pinMode(HS15_P_OUT, INPUT);             //ハイインピーダンスへ(入力ポート)
        digitalWrite(HS15_P_OUT, LOW);          //プルアップOFF
        j = analogRead(HS15_P_IN);              //A/D 10bit
    }
    interrupts();
    
    pinMode(HS15_P_OUT, OUTPUT);                //出力ポート動作へ
    digitalWrite(HS15_P_OUT, LOW);              //次の動作のために放電実行、放置
    return( j );                                //AD値を応答
}

float HS15Read( const float temp ){
    int i,j;
    unsigned int t;
    float Rs;                                   // HS15の抵抗値（対数）
    float Rh;                                   // 算出した湿度

    t=5;                                        // 最初は10uSec充電での測定から開始する
    for(j=0; j<5; j++){
        i = HS15GetRs(t);
        Rs = -1.0e-5 / 2.2e-8 / log( 1 - (float)i /1023 );  // 抵抗値算出
        Rs = -40 * log10( Rs / 3160 );                      // 対数抵抗値へ変換
        if( Rs > 0 ){
            Rs = 200 - (float)j * 40 + Rs;
            break;                              // forを抜ける
        }
        t *=10;                                 // 10倍のレンジでリトライ
    }
    //  温度補正(元のソースリストのまま使用)
    Rs += (Rs*35-13056) /64 *temp/64;           // ※特性は直線的でないので、
    if(Rs < 0 ) Rs= 0;                          // 単純にシフトするだけでは誤差が大きい
    //  リニアリティ補正(近似式：元のソースリストのまま使用)
    Rh = (5*Rs /16 *Rs + 44* Rs )/256 +20;
    return( Rh );
}
