/***************************************************************************************
受信したシリアル信号からの指示でベルを鳴らすXBee子機の実験用サンプルです。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/

#include <xbee.h>
#include <LiquidCrystal.h>
#define AEHA        0
#define NEC         1
#define SIRC        2

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

void setup(){
    ir_init();
    lcd.begin(16, 2);                               // LCDの初期化
    lcd.clear(); lcd.print("Example 60 IR");
    xbee_init( 0x00 );                              // XBeeの初期化
    xbee_atcb( 4 );                                 // ネットワーク初期化
    xbee_atnj(0);                                   // 孫機に対するジョイン拒否
    lcd.setCursor(0, 1);                            // 液晶の２行目に移動
    lcd.print("Commissioning");                     // 文字を表示
    while( xbee_atai() > 0x01 ){                    // ネットワーク参加状況を確認
        delay(3000);
        xbee_atcb(1);                               // ネットワーク参加ボタンを押下
    }
    lcd.clear();
}


void loop(){
    byte i;
    byte len=0;                                     // 信号長
    byte type = AEHA;                               // 赤外線方式
    XBEE_RESULT xbee_result;                        // 受信データ用

    byte data = xbee_rx_call( &xbee_result );
    if( xbee_result.MODE == MODE_UART){             // シリアルデータを受信
        if(xbee_result.DATA[0] == 0x1B){    // ESCコードが先頭だったとき
            if( xbee_result.DATA[1]==(byte)'I' &&
                xbee_result.DATA[2]==(byte)'R'){    // コマンドが「IR」の時
                type = xbee_result.DATA[3];
                len = xbee_result.DATA[4];
                if( len+4 <= data){
                    lcd.clear();
                    lcd.print("IR ");
                    switch(type){
                        case NEC:   lcd.print("NEC  "); break;
                        case SIRC:  lcd.print("SIRC "); break;
                        case AEHA:
                        default:    lcd.print("AEHA "); break;
                    }
                    lcd.print("len="); lcd.print(len);
                    lcd.setCursor(0, 1);
                    for(i=0;i<len;i++){
                        lcd.print(xbee_result.DATA[i+5]>>4,HEX);
                        lcd.print(xbee_result.DATA[i+5]%16,HEX);
                    }
                    ir_send( &xbee_result.DATA[5] , len , type );
                }else{
                    lcd.clear(); lcd.print("IR ERROR");
                }
            }
        }
    }
}

/* データ例
 ESC  'I'  'R'  type len  data....
{0x1B,0x49,0x52,0x00,0x06,0xAA,0x5A,0x8F,0x12,0x15,0xE1}    // AQUOS 音量を下げる
{0x1B,0x49,0x52,0x00,0x05,0x2C,0x52,0x09,0x2B,0x22};        // 照明の輝度を下げる
*/
