/***************************************************************************************
取得した情報をファイルに保存するロガーの製作
※DF ROBOT製の液晶がSDカードの信号と干渉するのでAdafruit製の液晶を使用しています。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
#include <SD.h>

#define PIN_SDCS    4                               // SDのチップセレクトCSポート番号

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
File file;                                          // SDファイルの定義

void setup() {
    lcd.begin(16, 2);                               // LCDのサイズを16文字×2桁に設定
    lcd.print("Example 61 LOG");                    // 文字を表示
    while(SD.begin(PIN_SDCS)==false){               // SDカードの開始
        delay(5000);                                // 失敗時は5秒ごとに繰り返し実施
    }
    xbee_init( 0 );                                 // XBee用COMポートの初期化
    xbee_atnj(0);                                   // 親機XBeeに子機の受け入れを拒否
}

void loop() {
    byte i,data;
    char s[17];
    
    float value;                                    // 受信データの代入用
    XBEE_RESULT xbee_result;                        // 受信データ(詳細)
    char  filename[]="01234567.CSV";                // 書き込みファイル名
    unsigned long time=millis()/1000;               // 時間[秒]の取得(約50日まで)

    if( lcd.readButtons()==BUTTON_DOWN ){           // 液晶シールドの下ボタンを押した時
        lcd.clear();                                // 液晶の文字を消去
        lcd.print("Waiting for XBee");              // コミッショニングボタン押下待ち
        xbee_atnj(15);                              // デバイスの参加受け入れを開始
        lcd.clear();
    }

    data=xbee_rx_call( &xbee_result );              // データを受信
    if( xbee_result.MODE == 0x00 ) return;          // データが無い時は終了

    for(i=0;i<4;i++){                               // 送信元アドレスをファイル名に設定
        filename[i*2]  =(char)(((xbee_result.FROM[i+4])>>4)+'0');
        filename[i*2+1]=(char)(((xbee_result.FROM[i+4])%16)+'0');
    }
    for(i=0;i<8;i++){
        if( filename[i] > '9') filename[i] += (char)('A'-'0'-10);
    }
    lcd.clear();
    lcd.print( time );                              // 時刻(起動からの経過時間)を表示
    lcd.setCursor(8, 0);                            // LCDカーソル位置を1行目右側へ
    lcd.print( filename );                          // ファイル名(送信元アドレス)を表示
    lcd.setCursor(0,1);
    file=SD.open(filename,FILE_WRITE);              // 書込みファイルのオープン
    file.print(time);                               // 時刻の出力
    if(file == true){                               // オープンが成功した場合
        switch( xbee_result.MODE ){
            case MODE_GPIN:                         // IOデータを受信した場合、
                for(i=1;i<=3;i++){                  // 　入力値をSDへ保存、液晶へ表示
                    file.print(", ");
                    if(xbee_result.ADCIN[i]==0xFFFF){
                        file.print((xbee_result.GPI.BYTE[0]>>i)&0x01,BIN);
                        lcd.print( (xbee_result.GPI.BYTE[0]>>i)&0x01,BIN);
                    }else{
                        file.print(xbee_result.ADCIN[i],DEC);
                        lcd.print(xbee_result.ADCIN[i],DEC);
                    }
                    lcd.print(" ");
                }
                break;
            case MODE_UART:                         // UARTシリアルデータを受信した場合
                for(i=0;i<=16;i++) s[i]='\0';       // 文字列変数sの初期化
                if(data > 16) data=16;              // 文字の扱いは16文字まで
                for(i=0;i<data;i++){                // 受信データを文字列変数sへ代入
                    if(isprint((char)xbee_result.DATA[i])){
                        s[i] = (char)xbee_result.DATA[i];
                    }else s[i] = ' ';
                }
                lcd.print( s );                     // 受信した文字列を表示
                file.print( s );                    // 受信した文字列をSDへ保存
                break;
            case MODE_IDNT:                         // コミッショニングボタン通知の時
                xbee_end_device(xbee_result.FROM,10,10,0);  // 自動測定10秒,S端子無効
                lcd.print("Found");                 // XBee子機の発見表示
                break;
        }
        file.println( "" );                         // SDへ改行を保存
        file.close();                               // ファイルクローズ
    }else{
        lcd.print("ERR");
    }
}
