/***************************************************************************************
組み込み版ポケットガイガーを使ったワイヤレス放射線センサを製作します。

                                                 Copyright (c) 2012-2013 Wataru KUNINO
                                                 http://www.geocities.jp/bokunimowakaru/
***************************************************************************************/

#include <xbee.h>
#include <Wire.h>
#include <DS3231.h>
#define     PIN_LED             13              // LED(SDのCLKと共通)
#define     PIN_XB_SLEEP        17              // Analag 3 XBeeのスリープ端子
#define     PIN_BATTERY         7               // バッテリ電圧入力(アナログ)
#define     TIME_MEAS           120             // 2分 測定時間(秒)
#define     SLEEP_DURATION      900             // 15分 休止間隔(秒)
DS3231 RTC;                                     // RTCオブジェクトの定義

byte dev[]={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}; // オール0はCoordinator
char s[40];                                     // 送信用の文字列
char sb[10];                                    // 文字列生成用のバッファ

void xbee_wake(){
    digitalWrite(PIN_XB_SLEEP, HIGH);           // XBeeのスリープ設定
    delay(10);
    digitalWrite(PIN_XB_SLEEP, LOW);            // XBeeのスリープ解除
    delay(10);
}

void xbee_setup(){
    xbee_at("ATST03E8");                        // Time before Sleep を1秒に設定
    xbee_at("ATSP012C");                        // 受信間隔を3秒に設定
    xbee_at("ATSM05");                          // Cyclic Sleep + PIN wakeモードに設定
}

void setup(){
    analogReference(INTERNAL);                  // ADCに内部の基準電圧を使用する
    pinMode(PIN_LED, OUTPUT);                   // LED(SDのCLKと共通)
    pinMode(PIN_XB_SLEEP, OUTPUT);              // XBeeのスリープ端子
    geiger_init();                              // ポケットガイガーの設定
    Wire.begin();
    RTC.begin();
    xbee_wake();                                // XBee ZBモジュールのスリープ解除
    xbee_init( 0x00 );                          // XBee用COMポートの初期化
    xbee_setup();                               // XBeeの省電力設定
    powMan_ei();
}

void loop(){
    int i,j=0;
    float temp;                                 // 温度
    float battery;                              // 電池電圧
    float num;                                  // カウント数の保存用
    float uSv;                                  // 放射線量uSv値の保存用
    DateTime now;                               // 現在時刻

    xbee_wake();                                // XBeeのスリープを解除する
    while( xbee_ping(dev) == 0xFF ){            // ping応答を確認
        while( xbee_atai() > 0x01 ){            // ネットワーク参加状況を確認
            for(i=0;i<60;i++){
                digitalWrite(PIN_LED,i%2);      // LEDの高速点滅
                delay(30);
            }
            xbee_wake();
            xbee_atcb(1);                       // ネットワーク参加ボタンを押下
            j++;
            if(j>10){
                j=0;
                xbee_atcb(4);                   // ネットワーク初期化
            }
        }
        xbee_setup();                           // XBeeの省電力設定
        delay(1000);
        powMan_ei();
    }
    xbee_from(dev);                             // ping応答のあったアドレスを保存
    xbee_atd(dev);                              // 宛先アドレスを設定
    geiger_on();                                // ポケットガイガーの電源をONにする

    /* 測定 */
    geiger_time_reset();                        // ガイガーカウント値のリセット
    while( geiger_time() <= TIME_MEAS ){        // 測定時間がTIME_MEAS内の時に繰り返し
        num=geiger_count_6sec();                // ガイガーカウント（測定に6秒を要する）
        uSv=geiger_uSv_6sec();                  // 放射線量の値を取得
        RTC.convertTemperature();               // 現在の温度値をレジスタに保存
        temp = RTC.getTemperature();            // RTCから温度の測定結果を読み込み
        battery = ((float)analogRead(PIN_BATTERY)-500)/160+3.2; // バッテリ電圧を取得
        now = RTC.now();                        // 現在の時刻を読み込む
        sprintf(s,"%02d%02d%02d",now.hour(),now.minute(),now.second());
        if( num < 1000 ) sprintf( sb ,",%03d", (int)num );
        else sprintf( sb ,",XXX" );
        strcat( s, sb );
        if( uSv < 10 ) sprintf( sb ,",%03d", (int)(uSv*100) );
        else sprintf( sb ,",XXX" );
        strcat( s, sb );
        sprintf( sb ,",%03d", (int)(battery*100) );
        strcat( s, sb );
        sprintf( sb ,",%03d\r", (int)(temp*10) );
        strcat( s, sb );
        xbee_wake();                            // XBee ZBモジュールのスリープ(一時)解除
        xbee_uart( dev , s);                    // 測定結果（文字列）の送信
        delay(100);                             // 送信完了待ち
        if( battery < 3.6 ) break;              // 電池電圧3.6V未満だとループを抜ける
    }
    digitalWrite(PIN_XB_SLEEP, LOW);            // XBeeのスリープ解除
    digitalWrite(PIN_LED, LOW);                 // LED消灯
    geiger_off();                               // ポケガの電源をOFFにする
    powMan_sleep();                             // スリープへ移行
}
