/***************************************************************************************
組み込み版ポケットガイガーを使ったワイヤレス放射線センサを製作します。

                                                 Copyright (c) 2012-2013 Wataru KUNINO
                                                 http://www.geocities.jp/bokunimowakaru/
----------------------------------------------------------------------------------------
ご注意：本スケッチの一部にRadiation-Watch.orgのソースコードが含まれます。

        Radiation-watch.orgプロジェクトは開発成果をクリエイティブ・コモンズ(CC 
        BY 3.0)ライセンスの元でオープンソース化しています。以下では、ポケット
        ガイガーの回路図、仕様、ソースコード、実験結果を、順次公開します。
        これらは全世界において、商用・非商用を問わず自由に再配布・改変・公開す
        ることができます。なお一部のコンテンツは英語またはオランダ語のみとなり
        ます。
        POCKET GEIGER by Radiation-Watch.org is licensed under a Creative 
        Commons Attribution 3.0 Unported License.

        Radiation-Watch.org
        URL http://www.radiation-watch.org/
***************************************************************************************/

#define     PIN_GEIGER          14              // Analag 0 ポケガの電源
#define     PIN_PULSE           15          // A1 ポケガ放射線パルス信号
#define     PIN_NOISE           16          // A2 ポケガノイズ検出信号
//#define     PIN_LED             13          // LED(SDのCLKと共通)
#define     ALPHA               53.032      	// uSvへの変換係数 cpm = uSv × alpha
// #define     PIN_NOISE_DISABLE

boolean pulse;                  // 放射線パルス信号の論理
boolean pulse_prev;             // 放射線パルス信号の前回値

int8_t time_prev;               // 前回時刻の秒
float count;                                  // 現在のカウント数
float uSvBuff;                                  // 換算値
int cpmTimeSec;                             // cpm計算用の時間[sec]
int count_err;


void geiger_on(){
	digitalWrite(PIN_GEIGER, LOW);              // ポケットガイガーの電源をONにする
    delay(1000);                                // 起動待ち（１秒間）
}

void geiger_off(){
	digitalWrite(PIN_GEIGER, HIGH);              // ポケットガイガーの電源をONにする
}

void geiger_init(){
    pinMode(PIN_GEIGER, OUTPUT);                // ポケガ用の電源制御ポートを出力に設定
    pinMode(PIN_PULSE, OUTPUT);                 // ポケガ放射線パルス信号入力を、一度、出力に
    digitalWrite(PIN_PULSE, HIGH);              // ポケガ放射線パルス信号入力を、一度、HIGH出力に
    pinMode(PIN_PULSE, INPUT_PULLUP);           // ポケガ放射線パルス信号入力を入力に
	geiger_off();
	pinMode(PIN_NOISE, OUTPUT);         // ポケガノイズ信号入力を、一度、出力に
	digitalWrite(PIN_NOISE, LOW);               // ポケガノイズ信号入力を、一度、LOW出力に
	delay(1000);
	pinMode(PIN_NOISE, INPUT);                  // ポケガノイズ検出信号
	digitalWrite(PIN_NOISE, HIGH);              // すぐにHIGH出力に。
	delay(500);									// (ディスチャージ)
	geiger_on();
    count = 0;
    count_err=0;
    cpmTimeSec = 0;
    time_prev = 99;
    pulse_prev = HIGH;
}

float geiger_count_6sec(){
    int i;
	int signCount=0;                  				// 信号のカウント回数
	int noiseCount=0;                 				// ノイズのカウント回数
    float min=0.0;									// 測定時間（分）
	DateTime now = RTC.now();						// RTCを主スケッチで定義しておくこと

    /* 測定 */
    do{
	    for( i=0 ; i<10000 ; i++){
	        pulse = digitalRead( PIN_PULSE );
	        if( pulse_prev != pulse ){
	            if( pulse == LOW )signCount++;          //放射線パルス(Low)をカウント
	            pulse_prev = pulse;
	        }
	        if( digitalRead( PIN_NOISE ) == HIGH ){
				#ifdef PIN_NOISE_DISABLE
				noiseCount=0;
				#else
	            noiseCount++;                           //ノイズ(High)をカウント
	            #endif
	        }
	    }
	    digitalWrite(PIN_LED, !(digitalRead( PIN_LED ))|(noiseCount!=0) );   // LED点滅
	    now = RTC.now();                                // get the current date-time
	}while( ( now.second() % 6 != 0 ) || (time_prev == now.second()) ); // 6秒ごとの処理
	
    time_prev = now.second() ;
    
    if(noiseCount == 0){                        // ノイズが検出されないとき
        count += (float)signCount;              // カウント回数の蓄積
        cpmTimeSec += 6 ;                       // cpmを求めるときに使用する計測時間
    }else{
		count_err++;
    	if( count_err > 3 ){
			geiger_off();
			pinMode(PIN_NOISE, OUTPUT);         // ポケガノイズ信号入力を、一度、出力に
			digitalWrite(PIN_NOISE, LOW);               // ポケガノイズ信号入力を、一度、LOW出力に
			delay(1000);
			pinMode(PIN_NOISE, INPUT);                  // ポケガノイズ検出信号
			digitalWrite(PIN_NOISE, HIGH);              // すぐにHIGH出力に。
			delay(500);									// (ディスチャージ)
			geiger_on();
		}
	}
    min = (float)cpmTimeSec / 60.0;             // 現在の計測時間(最大TIME_MEAS分)
    if(min != 0.) uSvBuff = count / min / ALPHA;
    else uSvBuff = 0;                           //0割り算のときは、0にする
	return(count);
}

float geiger_uSv_6sec(){
	return(uSvBuff);
}

int geiger_time(){
	return(cpmTimeSec);
}

void geiger_time_reset(){
    count = 0;
    count_err=0;
    cpmTimeSec = 0;
}


float geiger_count(){
	float ret;
	do{
		geiger_count_6sec();
	}while( cpmTimeSec <= TIME_MEAS );
	ret=count;
    count = 0;
    count_err=0;
    cpmTimeSec = 0;
	return(ret);
}

float geiger_uSv(){
	return(uSvBuff);
}


