/***************************************************************************************
XBeeセンサーから受信した内容をHTTPを使って表示します。(XML版)

                                                Copyright (c) 2010-2013 Wataru KUNINO
                                                http://www.geocities.jp/bokunimowakaru/
***************************************************************************************/
#include <xbee.h>
#include <SPI.h>
#include <Ethernet.h>

#define ADAFRUIT            // ← 液晶が DF ROBOT LCD Keypad Shieldの場合は本行を削除
#define DEVICE_N    2       // ← データを保持するXBee ZBデバイスの最大数
#define DAT_HIST_N  5       // ← 履歴データの最大数
byte mac[]={0x90,0xA2,0xDA,0x00,0x00,0x00}; // ← ArduinoのMACアドレス(裏面に記載)を転記
IPAddress ip(192,168,0,101);                // ← IPアドレスを設定

#ifndef ADAFRUIT
    #include <LiquidCrystalDFR.h>
    LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
#else
    #include <Wire.h>
    #include <Adafruit_MCP23017.h>
    #include <Adafruit_RGBLCDShield.h>
    Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
#endif

EthernetServer server(80);
EthernetClient client;
byte dev[DEVICE_N][8];                                  // デバイスのIEEEアドレス保持用
byte dev_en[DEVICE_N];                                  // 過去データの代入位置
byte dev_n = 0;                                         // 登録済みデバイスの数
char dat[DEVICE_N][DAT_HIST_N][17];                     // データ保持用
unsigned long dat_time[DEVICE_N][DAT_HIST_N];           // データの取得時刻の保持用

void lcd_cls( const byte line ){                        // 指定した行を消去する関数
    lcd.setCursor(0,line);
    for(byte i=0;i<16;i++)lcd.print(" ");
    lcd.setCursor(0,line);
}

void lcd_print_hex(const byte in){                      // 16進数2桁(1バイト)の表示関数
    lcd.print( in>>4 , HEX );
    lcd.print( in%16 , HEX );
}

byte xbee_address_lockup(byte *adr){                    // dev内で一致するアドレスを探す
    byte i,j;
    for(i=0;i<dev_n;i++){
        for(j=0;j<8;j++){
            if(dev[i][j]!=adr[j]) break;
        }
        if(j==8) break;
    }
    if(i==dev_n) i=0xFF;
    return(i);
}

void lcd_print_address(char *str, byte *dev ){          // アドレス表示用の関数
    byte i;
    byte start = 4;                                     // アドレスの表示開始バイト
    lcd_cls(1);                                         // 2行目を消去
    if( str[0] == '\0' ) start=0;                       // strに文字なし時は8バイト表示
    else lcd.print( str );                              // 文字あり時は文字+4バイト表示
    for(i=start;i<8;i++) lcd_print_hex(dev[i]);         // アドレスを液晶に表示する
}

void setup(){
    lcd.begin(16, 2);
    lcd.print("Gateway ZB to IP");                      // 文字を表示
    xbee_init( 0x00 );                                  // XBee用COMポートの初期化
    xbee_atnj(0);                                       // ジョイン拒否に設定
    Ethernet.begin(mac, ip);
    server.begin();
    lcd.setCursor(0,1);
    lcd.print(Ethernet.localIP());
}

void loop(){
    byte i,j,id;
    char c;
    byte data;
    char s[17];
    char buf[8];
    XBEE_RESULT xbee_result;                            // 受信データ用の構造体
    unsigned long time=millis()/1000;                   // 時間[秒]の取得(約50日まで)

    /* HTTPサーバ機能 */
    client = server.available();
    if(client == true){
        if(client.connected()){
            client.println("HTTP/1.1 200 OK");
            client.println("Content-Type: text/xml; charset=\"UTF-8\"");
            client.println("Connnection: close");
            client.println();
            client.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            client.println("<xbee>");
            if(dev_n !=0){
                for(id=0;id<dev_n;id++){
                    client.print("<device id=\"");
                    for(i=0;i<8;i++){
                        client.print( (dev[id][i])>>4 , HEX );
                        client.print( (dev[id][i])%16 , HEX );
                    }
                    client.println("\">");
                    i=dev_en[id];
                    j=0;
                    while( i<dev_en[id] || j==0 ){
                        client.print("<data time=\"");
                        sprintf(s,"%010u\">",dat_time[id][i]);
                        client.print(s);
                        if(dat[id][i][0]=='\0')client.print("null");
                        else client.print(dat[id][i]);
                        client.println("</data>");
                        i++;
                        if(i>=DAT_HIST_N){
                            j=1;
                            i=0;
                        }
                    }
                    client.println("</device>");
                }
            }
            client.println("</xbee>");
        }
        client.stop();
    }

    /* ボタン操作処理 */
    if( lcd.readButtons()==BUTTON_DOWN ){               // 液晶Shieldの下ボタンを押下時
        lcd_cls(0);                                     // 液晶の文字を消去
        lcd.print("Waiting for XBee");                  // コミッショニングボタン待ち
        xbee_atnj(0xFF);                                // デバイスの参加受け入れを開始
    }

    /* XBeeデータ受信処理 */
    data=xbee_rx_call( &xbee_result );
    if( xbee_result.MODE == 0x00 ||                     // データが無い時または
        xbee_result.MODE == MODE_RES ||                 // ATコマンド応答時または
        xbee_result.MODE == MODE_MODM ) return;         // モデム状態通知時は先頭へ戻る
    id=xbee_address_lockup(xbee_result.FROM);           // 受信したアドレスの照合
    lcd_cls(0);
    if(id == 0xFF){                                     // 履歴に無いアドレスを受信
        if(dev_n < DEVICE_N){
            for(i=0;i<8;i++) dev[dev_n][i]=xbee_result.FROM[i];
            dev_en[dev_n]=0;
            id=dev_n;
            dev_n++;
        }else{
            lcd.print("Unknown Devices");
            return;
        }
    }
    switch( xbee_result.MODE ){
        case MODE_GPIN:                                 // IOデータを受信した場合、
            s[0]='\0';                                  // 文字列変数sの初期化
            for(i=1;i<=3;i++){                          // 入力値をSDへ保存、液晶へ表示
                if(xbee_result.ADCIN[i] >= 1024){
                    sprintf(buf,"%01d,",(xbee_result.GPI.BYTE[0]>>i)&0x01);
                }else{
                    sprintf(buf,"%04d,",xbee_result.ADCIN[i]);
                }
                strcat(s,buf);
            }
            s[strlen(s)-1]='\0';                        // 末尾のコンマを消去
            lcd.print(s);
            lcd_print_address("Rx GPIN ",xbee_result.FROM);
            break;
        case MODE_UART:                                 // UARTシリアルを受信した場合
            for(i=0;i<=16;i++) s[i]='\0';               // 文字列変数sの初期化
            if(data > 16) data=16;                      // 文字の扱いは16文字まで
            for(i=0;i<data;i++){                        // 受信データを文字列変数sへ代入
                if(isprint((char)xbee_result.DATA[i])){
                    s[i] = (char)xbee_result.DATA[i];
                }else s[i] = ' ';
            }
            lcd.print(s);
            lcd_print_address("Rx UART ",xbee_result.FROM);
            break;
        case MODE_IDNT:                                 // コミッショニングボタン通知時
            xbee_end_device(xbee_result.FROM,10,10,0);  // 自動測定10秒,S端子無効
            xbee_atnj(0);                               // ジョイン拒否に設定
            lcd.print("Found a Dev No.");               // XBee子機の発見表示
            lcd.print(id,HEX);                          // 子機の管理番号の表示
            lcd_print_address("Rx IDNT ",xbee_result.FROM);
            break;
    }
    if( xbee_result.MODE == MODE_GPIN || xbee_result.MODE == MODE_UART ){
        strcpy(dat[id][dev_en[id]],s);                  // 文字列のコピー
        dat_time[id][dev_en[id]]=time;                  // 時刻のコピー
        dev_en[id]++;
        if(dev_en[id] >= DAT_HIST_N) dev_en[id]=0;
    }
}
